@@ -23,11 +23,8 @@
 #include "../include/ctdb_private.h"
 #include "../common/rb_tree.h"
 
-static bool is_child = false;
-
 void ctdb_set_child_info(TALLOC_CTX *mem_ctx, const char *child_name_fmt, ...)
 {
-	is_child = true;
 	if (child_name_fmt != NULL) {
 		va_list ap;
 		char *t;
@@ -40,11 +37,6 @@ void ctdb_set_child_info(TALLOC_CTX *mem_ctx, const char *child_name_fmt, ...)
 	}
 }
 
-bool ctdb_is_child_process(void)
-{
-	return is_child;
-}
-
 void ctdb_track_child(struct ctdb_context *ctdb, pid_t pid)
 {
 	char *process;
@@ -92,7 +84,7 @@ pid_t ctdb_fork_no_free_ringbuffer(struct ctdb_context *ctdb)
 
 		/* The child does not need to be realtime */
 		if (ctdb->do_setsched) {
-			ctdb_restore_scheduler(ctdb);
+			reset_scheduler();
 		}
 		ctdb->can_send_controls = false;
 
