@@ -44,6 +44,7 @@ struct ctdb_queue_pkt {
 	uint8_t *data;
 	uint32_t length;
 	uint32_t full_length;
+	uint8_t buf[];
 };
 
 struct ctdb_queue {
@@ -195,7 +196,9 @@ static void queue_io_read(struct ctdb_queue *queue)
 	}
 
 	if (num_ready > 0) {
-		nread = read(queue->fd, queue->buffer.data + queue->buffer.length, num_ready);
+		nread = sys_read(queue->fd,
+				 queue->buffer.data + queue->buffer.length,
+				 num_ready);
 		if (nread <= 0) {
 			DEBUG(DEBUG_ERR, ("read error nread=%d\n", (int)nread));
 			goto failed;
@@ -229,9 +232,9 @@ static void queue_io_write(struct ctdb_queue *queue)
 		struct ctdb_queue_pkt *pkt = queue->out_queue;
 		ssize_t n;
 		if (queue->ctdb->flags & CTDB_FLAG_TORTURE) {
-			n = write(queue->fd, pkt->data, 1);
+			n = sys_write(queue->fd, pkt->data, 1);
 		} else {
-			n = write(queue->fd, pkt->data, pkt->length);
+			n = sys_write(queue->fd, pkt->data, pkt->length);
 		}
 
 		if (n == -1 && errno != EAGAIN && errno != EWOULDBLOCK) {
@@ -285,6 +288,7 @@ static void queue_io_handler(struct event_context *ev, struct fd_event *fde,
 */
 int ctdb_queue_send(struct ctdb_queue *queue, uint8_t *data, uint32_t length)
 {
+	struct ctdb_req_header *hdr = (struct ctdb_req_header *)data;
 	struct ctdb_queue_pkt *pkt;
 	uint32_t length2, full_length;
 
@@ -306,7 +310,7 @@ int ctdb_queue_send(struct ctdb_queue *queue, uint8_t *data, uint32_t length)
 	   queue overhead. This relies on non-blocking sockets */
 	if (queue->out_queue == NULL && queue->fd != -1 &&
 	    !(queue->ctdb->flags & CTDB_FLAG_TORTURE)) {
-		ssize_t n = write(queue->fd, data, length2);
+		ssize_t n = sys_write(queue->fd, data, length2);
 		if (n == -1 && errno != EAGAIN && errno != EWOULDBLOCK) {
 			talloc_free(queue->fde);
 			queue->fde = NULL;
@@ -324,11 +328,13 @@ int ctdb_queue_send(struct ctdb_queue *queue, uint8_t *data, uint32_t length)
 		if (length2 == 0) return 0;
 	}
 
-	pkt = talloc(queue, struct ctdb_queue_pkt);
+	pkt = talloc_size(
+		queue, offsetof(struct ctdb_queue_pkt, buf) + length2);
 	CTDB_NO_MEMORY(queue->ctdb, pkt);
+	talloc_set_name_const(pkt, "struct ctdb_queue_pkt");
 
-	pkt->data = talloc_memdup(pkt, data, length2);
-	CTDB_NO_MEMORY(queue->ctdb, pkt->data);
+	pkt->data = pkt->buf;
+	memcpy(pkt->data, data, length2);
 
 	pkt->length = length2;
 	pkt->full_length = full_length;
@@ -342,7 +348,6 @@ int ctdb_queue_send(struct ctdb_queue *queue, uint8_t *data, uint32_t length)
 	queue->out_queue_length++;
 
 	if (queue->ctdb->tunable.verbose_memory_names != 0) {
-		struct ctdb_req_header *hdr = (struct ctdb_req_header *)pkt->data;
 		switch (hdr->operation) {
 		case CTDB_REQ_CONTROL: {
 			struct ctdb_req_control *c = (struct ctdb_req_control *)hdr;
