@@ -23,6 +23,9 @@
 #include "../include/ctdb_private.h"
 #include "../include/ctdb_client.h"
 
+int LogLevel = DEBUG_NOTICE;
+int this_log_level = 0;
+
 int log_ringbuf_size;
 
 #define MAX_LOG_SIZE 128
@@ -100,7 +103,8 @@ void ctdb_log_ringbuffer_free(void)
 	log_ringbuf_size = 0;
 }
 
-void ctdb_collect_log(struct ctdb_context *ctdb, struct ctdb_get_log_addr *log_addr)
+TDB_DATA ctdb_log_ringbuffer_collect_log(TALLOC_CTX *mem_ctx,
+					 enum debug_level max_level)
 {
 	TDB_DATA data;
 	FILE *f;
@@ -115,14 +119,15 @@ void ctdb_collect_log(struct ctdb_context *ctdb, struct ctdb_get_log_addr *log_a
 	/* dump to a file, then send the file as a blob */
 	f = tmpfile();
 	if (f == NULL) {
-		DEBUG(DEBUG_ERR,(__location__ " Unable to open tmpfile - %s\n", strerror(errno)));
-		return;
+		DEBUG(DEBUG_ERR,(__location__ " Unable to open tmpfile - %s\n",
+				 strerror(errno)));
+		return tdb_null;
 	}
 
 	for (i=0; i<ringbuf_count; i++) {
 		tmp_entry = (first_entry + i) % log_ringbuf_size;
 
-		if (log_entries[tmp_entry].level > log_addr->level) {
+		if (log_entries[tmp_entry].level > max_level) {
 		 	continue;
 		}
 
@@ -140,50 +145,21 @@ void ctdb_collect_log(struct ctdb_context *ctdb, struct ctdb_get_log_addr *log_a
 	if (fsize < 0) {
 		fclose(f);
 		DEBUG(DEBUG_ERR, ("Cannot get file size for log entries\n"));
-		return;
+		return tdb_null;
 	}
 	rewind(f);
 	data.dptr = talloc_size(NULL, fsize);
 	if (data.dptr == NULL) {
 		fclose(f);
-		CTDB_NO_MEMORY_VOID(ctdb, data.dptr);
+		DEBUG(DEBUG_ERR, (__location__ " Memory allocation error\n"));
+		return tdb_null;
 	}
 	data.dsize = fread(data.dptr, 1, fsize, f);
 	fclose(f);
 
 	DEBUG(DEBUG_ERR,("Marshalling log entries into a blob of %d bytes\n", (int)data.dsize));
 
-	DEBUG(DEBUG_ERR,("Send log to %d:%d\n", (int)log_addr->pnn, (int)log_addr->srvid));
-	ctdb_client_send_message(ctdb, log_addr->pnn, log_addr->srvid, data);
-
-	talloc_free(data.dptr);
-}
-
-int32_t ctdb_control_get_log(struct ctdb_context *ctdb, TDB_DATA addr)
-{
-	struct ctdb_get_log_addr *log_addr = (struct ctdb_get_log_addr *)addr.dptr;
-	pid_t child;
-
-	/* spawn a child process to marshall the huge log blob and send it back
-	   to the ctdb tool using a MESSAGE
-	*/
-	child = ctdb_fork_no_free_ringbuffer(ctdb);
-	if (child == (pid_t)-1) {
-		DEBUG(DEBUG_ERR,("Failed to fork a log collector child\n"));
-		return -1;
-	}
-
-	if (child == 0) {
-		ctdb_set_process_name("ctdb_log_collector");
-		if (switch_from_server_to_client(ctdb, "log-collector") != 0) {
-			DEBUG(DEBUG_CRIT, (__location__ "ERROR: failed to switch log collector child into client mode.\n"));
-			_exit(1);
-		}
-		ctdb_collect_log(ctdb, log_addr);
-		_exit(0);
-	}
-
-	return 0;
+	return data;
 }
 
 void ctdb_clear_log(struct ctdb_context *ctdb)
@@ -198,3 +174,40 @@ int32_t ctdb_control_clear_log(struct ctdb_context *ctdb)
 
 	return 0;
 }
+
+struct debug_levels debug_levels[] = {
+	{DEBUG_EMERG,	"EMERG"},
+	{DEBUG_ALERT,	"ALERT"},
+	{DEBUG_CRIT,	"CRIT"},
+	{DEBUG_ERR,	"ERR"},
+	{DEBUG_WARNING,	"WARNING"},
+	{DEBUG_NOTICE,	"NOTICE"},
+	{DEBUG_INFO,	"INFO"},
+	{DEBUG_DEBUG,	"DEBUG"},
+	{0, NULL}
+};
+
+const char *get_debug_by_level(int32_t level)
+{
+	int i;
+
+	for (i=0; debug_levels[i].description != NULL; i++) {
+		if (debug_levels[i].level == level) {
+			return debug_levels[i].description;
+		}
+	}
+	return "Unknown";
+}
+
+int32_t get_debug_by_desc(const char *desc)
+{
+	int i;
+
+	for (i=0; debug_levels[i].description != NULL; i++) {
+		if (!strcasecmp(debug_levels[i].description, desc)) {
+			return debug_levels[i].level;
+		}
+	}
+
+	return DEBUG_ERR;
+}
