@@ -198,16 +198,7 @@ static int32_t ctdb_control_dispatch(struct ctdb_context *ctdb,
 		return ctdb_control_push_db(ctdb, indata);
 
 	case CTDB_CONTROL_GET_RECMODE: {
-		int i;
-		if (ctdb->recovery_mode == CTDB_RECOVERY_ACTIVE) {
-			return CTDB_RECOVERY_ACTIVE;
-		}		  
-		for (i=1; i<=NUM_DB_PRIORITIES; i++) {
-			if (ctdb->freeze_mode[i] == CTDB_FREEZE_FROZEN) {
-				return CTDB_RECOVERY_ACTIVE;
-			}
-		}
-		return CTDB_RECOVERY_NORMAL;
+		return ctdb->recovery_mode;
 	}
 
 	case CTDB_CONTROL_SET_RECMASTER: {
@@ -327,7 +318,7 @@ static int32_t ctdb_control_dispatch(struct ctdb_context *ctdb,
 
 	case CTDB_CONTROL_THAW:
 		CHECK_CONTROL_DATA_SIZE(0);
-		return ctdb_control_thaw(ctdb, (uint32_t)c->srvid);
+		return ctdb_control_thaw(ctdb, (uint32_t)c->srvid, true);
 
 	case CTDB_CONTROL_SET_RECMODE:
 		CHECK_CONTROL_DATA_SIZE(sizeof(uint32_t));		
@@ -603,7 +594,7 @@ static int32_t ctdb_control_dispatch(struct ctdb_context *ctdb,
 
 	case CTDB_CONTROL_SET_DB_PRIORITY:
 		CHECK_CONTROL_DATA_SIZE(sizeof(struct ctdb_db_priority));
-		return ctdb_control_set_db_priority(ctdb, indata);
+		return ctdb_control_set_db_priority(ctdb, indata, client_id);
 
 	case CTDB_CONTROL_GET_DB_PRIORITY: {
 		uint32_t db_id;
@@ -682,6 +673,9 @@ static int32_t ctdb_control_dispatch(struct ctdb_context *ctdb,
 	case CTDB_CONTROL_RECEIVE_RECORDS:
 		return ctdb_control_receive_records(ctdb, indata, outdata);
 
+	case CTDB_CONTROL_DB_DETACH:
+		return ctdb_control_db_detach(ctdb, indata, client_id);
+
 	default:
 		DEBUG(DEBUG_CRIT,(__location__ " Unknown CTDB control opcode %u\n", opcode));
 		return -1;
