@@ -60,7 +60,7 @@ static void ctdb_time_tick(struct event_context *ev, struct timed_event *te,
 {
 	struct ctdb_context *ctdb = talloc_get_type(private_data, struct ctdb_context);
 
-	if (getpid() != ctdbd_pid) {
+	if (getpid() != ctdb->ctdbd_pid) {
 		return;
 	}
 
@@ -94,7 +94,7 @@ static void ctdb_start_periodic_events(struct ctdb_context *ctdb)
 	ctdb_start_time_tickd(ctdb);
 }
 
-static void block_signal(int signum)
+static void ignore_signal(int signum)
 {
 	struct sigaction act;
 
@@ -504,15 +504,12 @@ static int setup_deferred_fetch_locks(struct ctdb_db_context *ctdb_db, struct ct
 	uint32_t *k;
 	struct ctdb_deferred_fetch_queue *dfq;
 
-	k = talloc_zero_size(call, ((call->key.dsize + 3) & 0xfffffffc) + 4);
+	k = ctdb_key_to_idkey(call, call->key);
 	if (k == NULL) {
 		DEBUG(DEBUG_ERR,("Failed to allocate key for deferred fetch\n"));
 		return -1;
 	}
 
-	k[0] = (call->key.dsize + 3) / 4 + 1;
-	memcpy(&k[1], call->key.dptr, call->key.dsize);
-
 	dfq  = talloc(call, struct ctdb_deferred_fetch_queue);
 	if (dfq == NULL) {
 		DEBUG(DEBUG_ERR,("Failed to allocate key for deferred fetch queue structure\n"));
@@ -543,15 +540,12 @@ static int requeue_duplicate_fetch(struct ctdb_db_context *ctdb_db, struct ctdb_
 	struct ctdb_deferred_fetch_queue *dfq;
 	struct ctdb_deferred_fetch_call *dfc;
 
-	k = talloc_zero_size(c, ((key.dsize + 3) & 0xfffffffc) + 4);
+	k = ctdb_key_to_idkey(c, key);
 	if (k == NULL) {
 		DEBUG(DEBUG_ERR,("Failed to allocate key for deferred fetch\n"));
 		return -1;
 	}
 
-	k[0] = (key.dsize + 3) / 4 + 1;
-	memcpy(&k[1], key.dptr, key.dsize);
-
 	dfq = trbt_lookuparray32(ctdb_db->deferred_fetch, k[0], &k[0]);
 	if (dfq == NULL) {
 		talloc_free(k);
@@ -602,7 +596,7 @@ static void daemon_request_call_from_client(struct ctdb_client *client,
 	struct ctdb_daemon_packet_wrap *w;
 
 	CTDB_INCREMENT_STAT(ctdb, total_calls);
-	CTDB_DECREMENT_STAT(ctdb, pending_calls);
+	CTDB_INCREMENT_STAT(ctdb, pending_calls);
 
 	ctdb_db = find_ctdb_db(client->ctdb, c->db_id);
 	if (!ctdb_db) {
@@ -659,6 +653,7 @@ static void daemon_request_call_from_client(struct ctdb_client *client,
 			if (ret != 0) {
 				DEBUG(DEBUG_ERR,(__location__ " ctdb_ltdb_unlock() failed with error %d\n", ret));
 			}
+			CTDB_DECREMENT_STAT(ctdb, pending_calls);
 			return;
 		}
 	}
@@ -691,6 +686,7 @@ static void daemon_request_call_from_client(struct ctdb_client *client,
 		if (ctdb_add_revoke_deferred_call(ctdb, ctdb_db, key, (struct ctdb_req_header *)c, daemon_incoming_packet, client) != 0) {
 			ctdb_fatal(ctdb, "Failed to add deferred call for revoke child");
 		}
+		CTDB_DECREMENT_STAT(ctdb, pending_calls);
 		return;
 	}
 
@@ -712,6 +708,7 @@ static void daemon_request_call_from_client(struct ctdb_client *client,
 			ctdb_fatal(ctdb, "Failed to add deferred call for revoke child");
 		}
 
+		CTDB_DECREMENT_STAT(ctdb, pending_calls);
 		return;
 	}		
 
@@ -1070,8 +1067,10 @@ static void ctdb_tevent_trace(enum tevent_trace_point tp,
 {
 	struct timeval diff;
 	struct timeval now;
+	struct ctdb_context *ctdb =
+		talloc_get_type(private_data, struct ctdb_context);
 
-	if (getpid() != ctdbd_pid) {
+	if (getpid() != ctdb->ctdbd_pid) {
 		return;
 	}
 
@@ -1109,7 +1108,8 @@ static void ctdb_tevent_trace(enum tevent_trace_point tp,
 
 static void ctdb_remove_pidfile(void)
 {
-	if (ctdbd_pidfile != NULL && !ctdb_is_child_process()) {
+	/* Only the main ctdbd's PID matches the SID */
+	if (ctdbd_pidfile != NULL && getsid(0) == getpid()) {
 		if (unlink(ctdbd_pidfile) == 0) {
 			DEBUG(DEBUG_NOTICE, ("Removed PID file %s\n",
 					     ctdbd_pidfile));
@@ -1162,19 +1162,20 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork, bool use_syslog)
 	tdb_reopen_all(false);
 
 	if (do_fork) {
-		setsid();
+		if (setsid() == -1) {
+			ctdb_die(ctdb, "Failed to setsid()\n");
+		}
 		close(0);
 		if (open("/dev/null", O_RDONLY) != 0) {
 			DEBUG(DEBUG_ALERT,(__location__ " Failed to setup stdin on /dev/null\n"));
 			exit(11);
 		}
 	}
-	block_signal(SIGPIPE);
+	ignore_signal(SIGPIPE);
 
-	ctdbd_pid = getpid();
-	ctdb->ctdbd_pid = ctdbd_pid;
+	ctdb->ctdbd_pid = getpid();
 	DEBUG(DEBUG_ERR, ("Starting CTDBD (Version %s) as PID: %u\n",
-			  CTDB_VERSION_STRING, ctdbd_pid));
+			  CTDB_VERSION_STRING, ctdb->ctdbd_pid));
 	ctdb_create_pidfile(ctdb->ctdbd_pid);
 
 	/* Make sure we log something when the daemon terminates.
@@ -1185,7 +1186,10 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork, bool use_syslog)
 
 	if (ctdb->do_setsched) {
 		/* try to set us up as realtime */
-		ctdb_set_scheduler(ctdb);
+		if (!set_scheduler()) {
+			exit(1);
+		}
+		DEBUG(DEBUG_NOTICE, ("Set real-time scheduler priority\n"));
 	}
 
 	/* ensure the socket is deleted on exit of the daemon */
@@ -1197,7 +1201,7 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork, bool use_syslog)
 
 	ctdb->ev = event_context_init(NULL);
 	tevent_loop_allow_nesting(ctdb->ev);
-	tevent_set_trace_callback(ctdb->ev, ctdb_tevent_trace, NULL);
+	tevent_set_trace_callback(ctdb->ev, ctdb_tevent_trace, ctdb);
 	ret = ctdb_init_tevent_logging(ctdb);
 	if (ret != 0) {
 		DEBUG(DEBUG_ALERT,("Failed to initialize TEVENT logging\n"));
@@ -1317,8 +1321,8 @@ int ctdb_start_daemon(struct ctdb_context *ctdb, bool do_fork, bool use_syslog)
 		exit(1);
 	}
 
-	ctdb_lockdown_memory(ctdb);
-	  
+	lockdown_memory(ctdb->valgrinding);
+
 	/* go into a wait loop to allow other nodes to complete */
 	event_loop_wait(ctdb->ev);
 
