@@ -33,7 +33,7 @@
  * 2. Once the locks are obtained, signal parent process via fd.
  * 3. Invoke registered callback routine with locking status.
  * 4. If the child process cannot get locks within certain time,
- *    diagnose using /proc/locks and log warning message
+ *    execute an external script to debug.
  *
  * ctdb_lock_record()      - get a lock on a record
  * ctdb_lock_db()          - get a lock on a DB
@@ -43,9 +43,6 @@
  *  auto_mark              - whether to mark/unmark DBs in before/after callback
  */
 
-/* FIXME: Add a tunable max_lock_processes_per_db */
-#define MAX_LOCK_PROCESSES_PER_DB		(100)
-
 enum lock_type {
 	LOCK_RECORD,
 	LOCK_DB,
@@ -57,7 +54,7 @@ static const char * const lock_type_str[] = {
 	"lock_record",
 	"lock_db",
 	"lock_alldb_prio",
-	"lock_db",
+	"lock_alldb",
 };
 
 struct lock_request;
@@ -71,20 +68,18 @@ struct lock_context {
 	TDB_DATA key;
 	uint32_t priority;
 	bool auto_mark;
-	struct lock_request *req_queue;
+	struct lock_request *request;
 	pid_t child;
 	int fd[2];
 	struct tevent_fd *tfd;
 	struct tevent_timer *ttimer;
-	pid_t block_child;
-	int block_fd[2];
 	struct timeval start_time;
 	uint32_t key_hash;
+	bool can_schedule;
 };
 
 /* lock_request is the client specific part for a lock request */
 struct lock_request {
-	struct lock_request *next, *prev;
 	struct lock_context *lctx;
 	void (*callback)(void *, bool);
 	void *private_data;
@@ -277,21 +272,31 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb);
  */
 static int ctdb_lock_context_destructor(struct lock_context *lock_ctx)
 {
+	if (lock_ctx->request) {
+		lock_ctx->request->lctx = NULL;
+	}
 	if (lock_ctx->child > 0) {
 		ctdb_kill(lock_ctx->ctdb, lock_ctx->child, SIGKILL);
-		DLIST_REMOVE(lock_ctx->ctdb->lock_current, lock_ctx);
+		if (lock_ctx->type == LOCK_RECORD) {
+			DLIST_REMOVE(lock_ctx->ctdb_db->lock_current, lock_ctx);
+		} else {
+			DLIST_REMOVE(lock_ctx->ctdb->lock_current, lock_ctx);
+		}
 		if (lock_ctx->ctdb_db) {
 			lock_ctx->ctdb_db->lock_num_current--;
 		}
 		CTDB_DECREMENT_STAT(lock_ctx->ctdb, locks.num_current);
-		if (lock_ctx->type == LOCK_RECORD || lock_ctx->type == LOCK_DB) {
+		if (lock_ctx->ctdb_db) {
 			CTDB_DECREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_current);
 		}
 	} else {
-		DLIST_REMOVE(lock_ctx->ctdb->lock_pending, lock_ctx);
-		lock_ctx->ctdb->lock_num_pending--;
+		if (lock_ctx->type == LOCK_RECORD) {
+			DLIST_REMOVE(lock_ctx->ctdb_db->lock_pending, lock_ctx);
+		} else {
+			DLIST_REMOVE(lock_ctx->ctdb->lock_pending, lock_ctx);
+		}
 		CTDB_DECREMENT_STAT(lock_ctx->ctdb, locks.num_pending);
-		if (lock_ctx->type == LOCK_RECORD || lock_ctx->type == LOCK_DB) {
+		if (lock_ctx->ctdb_db) {
 			CTDB_DECREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_pending);
 		}
 	}
@@ -307,21 +312,10 @@ static int ctdb_lock_context_destructor(struct lock_context *lock_ctx)
  */
 static int ctdb_lock_request_destructor(struct lock_request *lock_request)
 {
-	DLIST_REMOVE(lock_request->lctx->req_queue, lock_request);
+	TALLOC_FREE(lock_request->lctx);
 	return 0;
 }
 
-
-void ctdb_lock_free_request_context(struct lock_request *lock_req)
-{
-	struct lock_context *lock_ctx;
-
-	lock_ctx = lock_req->lctx;
-	talloc_free(lock_req);
-	talloc_free(lock_ctx);
-}
-
-
 /*
  * Process all the callbacks waiting for lock
  *
@@ -329,7 +323,7 @@ void ctdb_lock_free_request_context(struct lock_request *lock_req)
  */
 static void process_callbacks(struct lock_context *lock_ctx, bool locked)
 {
-	struct lock_request *request, *next;
+	struct lock_request *request;
 
 	if (lock_ctx->auto_mark && locked) {
 		switch (lock_ctx->type) {
@@ -351,19 +345,12 @@ static void process_callbacks(struct lock_context *lock_ctx, bool locked)
 		}
 	}
 
-	/* Iterate through all callbacks */
-	request = lock_ctx->req_queue;
-	while (request) {
-		if (lock_ctx->auto_mark) {
-			/* Reset the destructor, so request is not removed from the list */
-			talloc_set_destructor(request, NULL);
-		}
-
-		/* In case, callback frees the request, store next */
-		next = request->next;
-		request->callback(request->private_data, locked);
-		request = next;
+	request = lock_ctx->request;
+	if (lock_ctx->auto_mark) {
+		/* Reset the destructor, so request is not removed from the list */
+		talloc_set_destructor(request, NULL);
 	}
+	request->callback(request->private_data, locked);
 
 	if (lock_ctx->auto_mark && locked) {
 		switch (lock_ctx->type) {
@@ -438,9 +425,7 @@ static void ctdb_lock_handler(struct tevent_context *ev,
 	lock_ctx = talloc_get_type_abort(private_data, struct lock_context);
 
 	/* cancel the timeout event */
-	if (lock_ctx->ttimer) {
-		TALLOC_FREE(lock_ctx->ttimer);
-	}
+	TALLOC_FREE(lock_ctx->ttimer);
 
 	t = timeval_elapsed(&lock_ctx->start_time);
 	id = lock_bucket_id(t);
@@ -451,17 +436,15 @@ static void ctdb_lock_handler(struct tevent_context *ev,
 	}
 
 	/* Read the status from the child process */
-	if (read(lock_ctx->fd[0], &c, 1) != 1) {
+	if (sys_read(lock_ctx->fd[0], &c, 1) != 1) {
 		locked = false;
 	} else {
 		locked = (c == 0 ? true : false);
 	}
 
 	/* Update statistics */
-	CTDB_DECREMENT_STAT(lock_ctx->ctdb, locks.num_pending);
 	CTDB_INCREMENT_STAT(lock_ctx->ctdb, locks.num_calls);
 	if (lock_ctx->ctdb_db) {
-		CTDB_DECREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_pending);
 		CTDB_INCREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_calls);
 	}
 
@@ -507,7 +490,12 @@ static void ctdb_lock_timeout_handler(struct tevent_context *ev,
 	lock_ctx = talloc_get_type_abort(private_data, struct lock_context);
 	ctdb = lock_ctx->ctdb;
 
-	if (lock_ctx->type == LOCK_RECORD || lock_ctx->type == LOCK_DB) {
+	/* If a node stopped/banned, don't spam the logs */
+	if (ctdb->nodes[ctdb->pnn]->flags & NODE_FLAGS_INACTIVE) {
+		lock_ctx->ttimer = NULL;
+		return;
+	}
+	if (lock_ctx->ctdb_db) {
 		DEBUG(DEBUG_WARNING,
 		      ("Unable to get %s lock on database %s for %.0lf seconds\n",
 		       (lock_ctx->type == LOCK_RECORD ? "RECORD" : "DB"),
@@ -562,7 +550,7 @@ static int db_count_handler(struct ctdb_db_context *ctdb_db, uint32_t priority,
 }
 
 struct db_namelist {
-	char **names;
+	const char **names;
 	int n;
 };
 
@@ -577,30 +565,32 @@ static int db_name_handler(struct ctdb_db_context *ctdb_db, uint32_t priority,
 	return 0;
 }
 
-static char **lock_helper_args(TALLOC_CTX *mem_ctx, struct lock_context *lock_ctx, int fd)
+static bool lock_helper_args(TALLOC_CTX *mem_ctx,
+			     struct lock_context *lock_ctx, int fd,
+			     int *argc, const char ***argv)
 {
 	struct ctdb_context *ctdb = lock_ctx->ctdb;
-	char **args = NULL;
+	const char **args = NULL;
 	int nargs, i;
 	int priority;
 	struct db_namelist list;
 
 	switch (lock_ctx->type) {
 	case LOCK_RECORD:
-		nargs = 6;
-		break;
-
-	case LOCK_DB:
 		nargs = 5;
 		break;
 
-	case LOCK_ALLDB_PRIO:
+	case LOCK_DB:
 		nargs = 4;
+		break;
+
+	case LOCK_ALLDB_PRIO:
+		nargs = 3;
 		ctdb_db_iterator(ctdb, lock_ctx->priority, db_count_handler, &nargs);
 		break;
 
 	case LOCK_ALLDB:
-		nargs = 4;
+		nargs = 3;
 		for (priority=1; priority<NUM_DB_PRIORITIES; priority++) {
 			ctdb_db_iterator(ctdb, priority, db_count_handler, &nargs);
 		}
@@ -610,42 +600,41 @@ static char **lock_helper_args(TALLOC_CTX *mem_ctx, struct lock_context *lock_ct
 	/* Add extra argument for null termination */
 	nargs++;
 
-	args = talloc_array(mem_ctx, char *, nargs);
+	args = talloc_array(mem_ctx, const char *, nargs);
 	if (args == NULL) {
-		return NULL;
+		return false;
 	}
 
-	args[0] = talloc_strdup(args, "ctdb_lock_helper");
-	args[1] = talloc_asprintf(args, "%d", getpid());
-	args[2] = talloc_asprintf(args, "%d", fd);
+	args[0] = talloc_asprintf(args, "%d", getpid());
+	args[1] = talloc_asprintf(args, "%d", fd);
 
 	switch (lock_ctx->type) {
 	case LOCK_RECORD:
-		args[3] = talloc_strdup(args, "RECORD");
-		args[4] = talloc_strdup(args, lock_ctx->ctdb_db->db_path);
+		args[2] = talloc_strdup(args, "RECORD");
+		args[3] = talloc_strdup(args, lock_ctx->ctdb_db->db_path);
 		if (lock_ctx->key.dsize == 0) {
-			args[5] = talloc_strdup(args, "NULL");
+			args[4] = talloc_strdup(args, "NULL");
 		} else {
-			args[5] = hex_encode_talloc(args, lock_ctx->key.dptr, lock_ctx->key.dsize);
+			args[4] = hex_encode_talloc(args, lock_ctx->key.dptr, lock_ctx->key.dsize);
 		}
 		break;
 
 	case LOCK_DB:
-		args[3] = talloc_strdup(args, "DB");
-		args[4] = talloc_strdup(args, lock_ctx->ctdb_db->db_path);
+		args[2] = talloc_strdup(args, "DB");
+		args[3] = talloc_strdup(args, lock_ctx->ctdb_db->db_path);
 		break;
 
 	case LOCK_ALLDB_PRIO:
-		args[3] = talloc_strdup(args, "DB");
+		args[2] = talloc_strdup(args, "DB");
 		list.names = args;
-		list.n = 4;
+		list.n = 3;
 		ctdb_db_iterator(ctdb, lock_ctx->priority, db_name_handler, &list);
 		break;
 
 	case LOCK_ALLDB:
-		args[3] = talloc_strdup(args, "DB");
+		args[2] = talloc_strdup(args, "DB");
 		list.names = args;
-		list.n = 4;
+		list.n = 3;
 		for (priority=1; priority<NUM_DB_PRIORITIES; priority++) {
 			ctdb_db_iterator(ctdb, priority, db_name_handler, &list);
 		}
@@ -658,79 +647,86 @@ static char **lock_helper_args(TALLOC_CTX *mem_ctx, struct lock_context *lock_ct
 	for (i=0; i<nargs-1; i++) {
 		if (args[i] == NULL) {
 			talloc_free(args);
-			return NULL;
+			return false;
 		}
 	}
 
-	return args;
+	*argc = nargs;
+	*argv = args;
+	return true;
 }
 
-
 /*
- * Find the lock context of a given type
+ * Find a lock request that can be scheduled
  */
-static struct lock_context *find_lock_context(struct lock_context *lock_list,
-					      struct ctdb_db_context *ctdb_db,
-					      TDB_DATA key,
-					      uint32_t priority,
-					      enum lock_type type,
-					      uint32_t key_hash)
+struct lock_context *ctdb_find_lock_context(struct ctdb_context *ctdb)
 {
-	struct lock_context *lock_ctx;
+	struct lock_context *lock_ctx, *next_ctx;
+	struct ctdb_db_context *ctdb_db;
 
-	/* Search active locks */
-	for (lock_ctx=lock_list; lock_ctx; lock_ctx=lock_ctx->next) {
-		if (lock_ctx->type != type) {
+	/* First check if there are database lock requests */
+
+	for (lock_ctx = ctdb->lock_pending; lock_ctx != NULL;
+	     lock_ctx = next_ctx) {
+
+		if (lock_ctx->request != NULL) {
+			/* Found a lock context with a request */
+			return lock_ctx;
+		}
+
+		next_ctx = lock_ctx->next;
+
+		DEBUG(DEBUG_INFO, ("Removing lock context without lock "
+				   "request\n"));
+		DLIST_REMOVE(ctdb->lock_pending, lock_ctx);
+		CTDB_DECREMENT_STAT(ctdb, locks.num_pending);
+		if (lock_ctx->ctdb_db) {
+			CTDB_DECREMENT_DB_STAT(lock_ctx->ctdb_db,
+					       locks.num_pending);
+		}
+		talloc_free(lock_ctx);
+	}
+
+	/* Next check database queues */
+	for (ctdb_db = ctdb->db_list; ctdb_db; ctdb_db = ctdb_db->next) {
+		if (ctdb_db->lock_num_current ==
+		    ctdb->tunable.lock_processes_per_db) {
 			continue;
 		}
 
-		switch (lock_ctx->type) {
-		case LOCK_RECORD:
-			if (ctdb_db == lock_ctx->ctdb_db &&
-			    key_hash == lock_ctx->key_hash) {
-				goto done;
-			}
-			break;
+		for (lock_ctx = ctdb_db->lock_pending; lock_ctx != NULL;
+		     lock_ctx = next_ctx) {
 
-		case LOCK_DB:
-			if (ctdb_db == lock_ctx->ctdb_db) {
-				goto done;
-			}
-			break;
+			next_ctx = lock_ctx->next;
 
-		case LOCK_ALLDB_PRIO:
-			if (priority == lock_ctx->priority) {
-				goto done;
+			if (lock_ctx->request != NULL) {
+				return lock_ctx;
 			}
-			break;
 
-		case LOCK_ALLDB:
-			goto done;
-			break;
+			DEBUG(DEBUG_INFO, ("Removing lock context without "
+					   "lock request\n"));
+			DLIST_REMOVE(ctdb_db->lock_pending, lock_ctx);
+			CTDB_DECREMENT_STAT(ctdb, locks.num_pending);
+			CTDB_DECREMENT_DB_STAT(ctdb_db, locks.num_pending);
+			talloc_free(lock_ctx);
 		}
 	}
 
-	/* Did not find the lock context we are searching for */
-	lock_ctx = NULL;
-
-done:
-	return lock_ctx;
-
+	return NULL;
 }
 
-
 /*
  * Schedule a new lock child process
  * Set up callback handler and timeout handler
  */
 static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 {
-	struct lock_context *lock_ctx, *next_ctx, *active_ctx;
-	int ret;
+	struct lock_context *lock_ctx;
+	int ret, argc;
 	TALLOC_CTX *tmp_ctx;
 	const char *helper = BINDIR "/ctdb_lock_helper";
 	static const char *prog = NULL;
-	char **args;
+	const char **args;
 
 	if (prog == NULL) {
 		const char *t;
@@ -744,43 +740,8 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 		CTDB_NO_MEMORY_VOID(ctdb, prog);
 	}
 
-	if (ctdb->lock_pending == NULL) {
-		return;
-	}
-
 	/* Find a lock context with requests */
-	lock_ctx = ctdb->lock_pending;
-	while (lock_ctx != NULL) {
-		next_ctx = lock_ctx->next;
-		if (! lock_ctx->req_queue) {
-			DEBUG(DEBUG_INFO, ("Removing lock context without lock requests\n"));
-			DLIST_REMOVE(ctdb->lock_pending, lock_ctx);
-			ctdb->lock_num_pending--;
-			CTDB_DECREMENT_STAT(ctdb, locks.num_pending);
-			if (lock_ctx->ctdb_db) {
-				CTDB_DECREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_pending);
-			}
-			talloc_free(lock_ctx);
-		} else {
-			active_ctx = find_lock_context(ctdb->lock_current, lock_ctx->ctdb_db,
-						       lock_ctx->key, lock_ctx->priority,
-						       lock_ctx->type, lock_ctx->key_hash);
-			if (active_ctx == NULL) {
-				if (lock_ctx->ctdb_db == NULL ||
-				    lock_ctx->ctdb_db->lock_num_current < MAX_LOCK_PROCESSES_PER_DB) {
-					/* Found a lock context with lock requests */
-					break;
-				}
-			}
-
-			/* There is already a child waiting for the
-			 * same key.  So don't schedule another child
-			 * just yet.
-			 */
-		}
-		lock_ctx = next_ctx;
-	}
-
+	lock_ctx = ctdb_find_lock_context(ctdb);
 	if (lock_ctx == NULL) {
 		return;
 	}
@@ -804,8 +765,8 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 	}
 
 	/* Create arguments for lock helper */
-	args = lock_helper_args(tmp_ctx, lock_ctx, lock_ctx->fd[1]);
-	if (args == NULL) {
+	if (!lock_helper_args(tmp_ctx, lock_ctx, lock_ctx->fd[1],
+			      &argc, &args)) {
 		DEBUG(DEBUG_ERR, ("Failed to create lock helper args\n"));
 		close(lock_ctx->fd[0]);
 		close(lock_ctx->fd[1]);
@@ -813,9 +774,9 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 		return;
 	}
 
-	lock_ctx->child = vfork();
-
-	if (lock_ctx->child == (pid_t)-1) {
+	if (!ctdb_vfork_with_logging(lock_ctx, ctdb, "lock_helper",
+				     prog, argc, (const char **)args,
+				     NULL, NULL, &lock_ctx->child)) {
 		DEBUG(DEBUG_ERR, ("Failed to create a child in ctdb_lock_schedule\n"));
 		close(lock_ctx->fd[0]);
 		close(lock_ctx->fd[1]);
@@ -823,19 +784,7 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 		return;
 	}
 
-
-	/* Child process */
-	if (lock_ctx->child == 0) {
-		ret = execv(prog, args);
-		if (ret < 0) {
-			DEBUG(DEBUG_ERR, ("Failed to execute helper %s (%d, %s)\n",
-					  prog, errno, strerror(errno)));
-		}
-		_exit(1);
-	}
-
 	/* Parent process */
-	ctdb_track_child(ctdb, lock_ctx->child);
 	close(lock_ctx->fd[1]);
 
 	talloc_set_destructor(lock_ctx, ctdb_lock_context_destructor);
@@ -874,12 +823,18 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 	tevent_fd_set_auto_close(lock_ctx->tfd);
 
 	/* Move the context from pending to current */
-	DLIST_REMOVE(ctdb->lock_pending, lock_ctx);
-	ctdb->lock_num_pending--;
-	DLIST_ADD_END(ctdb->lock_current, lock_ctx, NULL);
+	if (lock_ctx->type == LOCK_RECORD) {
+		DLIST_REMOVE(lock_ctx->ctdb_db->lock_pending, lock_ctx);
+		DLIST_ADD_END(lock_ctx->ctdb_db->lock_current, lock_ctx, NULL);
+	} else {
+		DLIST_REMOVE(ctdb->lock_pending, lock_ctx);
+		DLIST_ADD_END(ctdb->lock_current, lock_ctx, NULL);
+	}
+	CTDB_DECREMENT_STAT(lock_ctx->ctdb, locks.num_pending);
+	CTDB_INCREMENT_STAT(lock_ctx->ctdb, locks.num_current);
 	if (lock_ctx->ctdb_db) {
 		lock_ctx->ctdb_db->lock_num_current++;
-		CTDB_INCREMENT_STAT(lock_ctx->ctdb, locks.num_current);
+		CTDB_DECREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_pending);
 		CTDB_INCREMENT_DB_STAT(lock_ctx->ctdb_db, locks.num_current);
 	}
 }
@@ -888,7 +843,8 @@ static void ctdb_lock_schedule(struct ctdb_context *ctdb)
 /*
  * Lock record / db depending on type
  */
-static struct lock_request *ctdb_lock_internal(struct ctdb_context *ctdb,
+static struct lock_request *ctdb_lock_internal(TALLOC_CTX *mem_ctx,
+					       struct ctdb_context *ctdb,
 					       struct ctdb_db_context *ctdb_db,
 					       TDB_DATA key,
 					       uint32_t priority,
@@ -905,66 +861,59 @@ static struct lock_request *ctdb_lock_internal(struct ctdb_context *ctdb,
 		return NULL;
 	}
 
-#if 0
-	/* Disable this optimization to ensure first-in-first-out fair
-	 * scheduling of lock requests */
-
-	/* get a context for this key - search only the pending contexts,
-	 * current contexts might in the middle of processing callbacks */
-	lock_ctx = find_lock_context(ctdb->lock_pending, ctdb_db, key, priority, type);
-#endif
-
-	/* No existing context, create one */
+	lock_ctx = talloc_zero(ctdb, struct lock_context);
 	if (lock_ctx == NULL) {
-		lock_ctx = talloc_zero(ctdb, struct lock_context);
-		if (lock_ctx == NULL) {
-			DEBUG(DEBUG_ERR, ("Failed to create a new lock context\n"));
-			return NULL;
-		}
-
-		lock_ctx->type = type;
-		lock_ctx->ctdb = ctdb;
-		lock_ctx->ctdb_db = ctdb_db;
-		lock_ctx->key.dsize = key.dsize;
-		if (key.dsize > 0) {
-			lock_ctx->key.dptr = talloc_memdup(lock_ctx, key.dptr, key.dsize);
-			if (lock_ctx->key.dptr == NULL) {
-				DEBUG(DEBUG_ERR, (__location__ "Memory allocation error\n"));
-				talloc_free(lock_ctx);
-				return NULL;
-			}
-			lock_ctx->key_hash = ctdb_hash(&key);
-		} else {
-			lock_ctx->key.dptr = NULL;
-		}
-		lock_ctx->priority = priority;
-		lock_ctx->auto_mark = auto_mark;
-
-		lock_ctx->child = -1;
-		lock_ctx->block_child = -1;
-
-		DLIST_ADD_END(ctdb->lock_pending, lock_ctx, NULL);
-		ctdb->lock_num_pending++;
-		CTDB_INCREMENT_STAT(ctdb, locks.num_pending);
-		if (ctdb_db) {
-			CTDB_INCREMENT_DB_STAT(ctdb_db, locks.num_pending);
-		}
-
-		/* Start the timer when we activate the context */
-		lock_ctx->start_time = timeval_current();
+		DEBUG(DEBUG_ERR, ("Failed to create a new lock context\n"));
+		return NULL;
 	}
 
-	if ((request = talloc_zero(lock_ctx, struct lock_request)) == NULL) {
+	if ((request = talloc_zero(mem_ctx, struct lock_request)) == NULL) {
 		talloc_free(lock_ctx);
 		return NULL;
 	}
 
+	lock_ctx->type = type;
+	lock_ctx->ctdb = ctdb;
+	lock_ctx->ctdb_db = ctdb_db;
+	lock_ctx->key.dsize = key.dsize;
+	if (key.dsize > 0) {
+		lock_ctx->key.dptr = talloc_memdup(lock_ctx, key.dptr, key.dsize);
+		if (lock_ctx->key.dptr == NULL) {
+			DEBUG(DEBUG_ERR, (__location__ "Memory allocation error\n"));
+			talloc_free(lock_ctx);
+			return NULL;
+		}
+		lock_ctx->key_hash = ctdb_hash(&key);
+	} else {
+		lock_ctx->key.dptr = NULL;
+	}
+	lock_ctx->priority = priority;
+	lock_ctx->auto_mark = auto_mark;
+
+	lock_ctx->request = request;
+	lock_ctx->child = -1;
+
+	/* Non-record locks are required by recovery and should be scheduled
+	 * immediately, so keep them at the head of the pending queue.
+	 */
+	if (lock_ctx->type == LOCK_RECORD) {
+		DLIST_ADD_END(ctdb_db->lock_pending, lock_ctx, NULL);
+	} else {
+		DLIST_ADD_END(ctdb->lock_pending, lock_ctx, NULL);
+	}
+	CTDB_INCREMENT_STAT(ctdb, locks.num_pending);
+	if (ctdb_db) {
+		CTDB_INCREMENT_DB_STAT(ctdb_db, locks.num_pending);
+	}
+
+	/* Start the timer when we activate the context */
+	lock_ctx->start_time = timeval_current();
+
 	request->lctx = lock_ctx;
 	request->callback = callback;
 	request->private_data = private_data;
 
 	talloc_set_destructor(request, ctdb_lock_request_destructor);
-	DLIST_ADD_END(lock_ctx->req_queue, request, NULL);
 
 	ctdb_lock_schedule(ctdb);
 
@@ -975,13 +924,15 @@ static struct lock_request *ctdb_lock_internal(struct ctdb_context *ctdb,
 /*
  * obtain a lock on a record in a database
  */
-struct lock_request *ctdb_lock_record(struct ctdb_db_context *ctdb_db,
+struct lock_request *ctdb_lock_record(TALLOC_CTX *mem_ctx,
+				      struct ctdb_db_context *ctdb_db,
 				      TDB_DATA key,
 				      bool auto_mark,
 				      void (*callback)(void *, bool),
 				      void *private_data)
 {
-	return ctdb_lock_internal(ctdb_db->ctdb,
+	return ctdb_lock_internal(mem_ctx,
+				  ctdb_db->ctdb,
 				  ctdb_db,
 				  key,
 				  0,
@@ -995,12 +946,14 @@ struct lock_request *ctdb_lock_record(struct ctdb_db_context *ctdb_db,
 /*
  * obtain a lock on a database
  */
-struct lock_request *ctdb_lock_db(struct ctdb_db_context *ctdb_db,
+struct lock_request *ctdb_lock_db(TALLOC_CTX *mem_ctx,
+				  struct ctdb_db_context *ctdb_db,
 				  bool auto_mark,
 				  void (*callback)(void *, bool),
 				  void *private_data)
 {
-	return ctdb_lock_internal(ctdb_db->ctdb,
+	return ctdb_lock_internal(mem_ctx,
+				  ctdb_db->ctdb,
 				  ctdb_db,
 				  tdb_null,
 				  0,
@@ -1014,7 +967,8 @@ struct lock_request *ctdb_lock_db(struct ctdb_db_context *ctdb_db,
 /*
  * obtain locks on all databases of specified priority
  */
-struct lock_request *ctdb_lock_alldb_prio(struct ctdb_context *ctdb,
+struct lock_request *ctdb_lock_alldb_prio(TALLOC_CTX *mem_ctx,
+					  struct ctdb_context *ctdb,
 					  uint32_t priority,
 					  bool auto_mark,
 					  void (*callback)(void *, bool),
@@ -1025,7 +979,8 @@ struct lock_request *ctdb_lock_alldb_prio(struct ctdb_context *ctdb,
 		return NULL;
 	}
 
-	return ctdb_lock_internal(ctdb,
+	return ctdb_lock_internal(mem_ctx,
+				  ctdb,
 				  NULL,
 				  tdb_null,
 				  priority,
@@ -1039,12 +994,14 @@ struct lock_request *ctdb_lock_alldb_prio(struct ctdb_context *ctdb,
 /*
  * obtain locks on all databases
  */
-struct lock_request *ctdb_lock_alldb(struct ctdb_context *ctdb,
+struct lock_request *ctdb_lock_alldb(TALLOC_CTX *mem_ctx,
+				     struct ctdb_context *ctdb,
 				     bool auto_mark,
 				     void (*callback)(void *, bool),
 				     void *private_data)
 {
-	return ctdb_lock_internal(ctdb,
+	return ctdb_lock_internal(mem_ctx,
+				  ctdb,
 				  NULL,
 				  tdb_null,
 				  0,
