@@ -129,7 +129,7 @@ int start_syslog_daemon(struct ctdb_context *ctdb)
 		set_close_on_exec(state->fd[0]);
 
 		close(startup_fd[1]);
-		n = read(startup_fd[0], &dummy, sizeof(dummy));
+		n = sys_read(startup_fd[0], &dummy, sizeof(dummy));
 		close(startup_fd[0]);
 		if (n < sizeof(dummy)) {
 			return -1;
@@ -182,7 +182,7 @@ int start_syslog_daemon(struct ctdb_context *ctdb)
 
 	/* Tell parent that we're up */
 	ret = 0;
-	write(startup_fd[1], &ret, sizeof(ret));
+	sys_write(startup_fd[1], &ret, sizeof(ret));
 	close(startup_fd[1]);
 
 	event_loop_wait(ctdb->ev);
@@ -304,7 +304,7 @@ static void ctdb_logfile_log(const char *format, va_list ap)
 	if (ret == -1) {
 		const char *errstr = "vasprintf failed\n";
 
-		write(log_state->fd, errstr, strlen(errstr));
+		sys_write(log_state->fd, errstr, strlen(errstr));
 		return;
 	}
 
@@ -319,11 +319,11 @@ static void ctdb_logfile_log(const char *format, va_list ap)
 	free(s);
 	if (ret == -1) {
 		const char *errstr = "asprintf failed\n";
-		write(log_state->fd, errstr, strlen(errstr));
+		sys_write(log_state->fd, errstr, strlen(errstr));
 		return;
 	}
 	if (s2) {
-		write(log_state->fd, s2, strlen(s2));
+		sys_write(log_state->fd, s2, strlen(s2));
 		free(s2);
 	}
 }
@@ -337,12 +337,12 @@ static void ctdb_logfile_log_add(const char *format, va_list ap)
 	if (ret == -1) {
 		const char *errstr = "vasprintf failed\n";
 
-		write(log_state->fd, errstr, strlen(errstr));
+		sys_write(log_state->fd, errstr, strlen(errstr));
 		return;
 	}
 
 	if (s) {
-		write(log_state->fd, s, strlen(s));
+		sys_write(log_state->fd, s, strlen(s));
 		free(s);
 	}
 }
@@ -424,7 +424,7 @@ static void ctdb_log_handler(struct event_context *ev, struct fd_event *fde,
 		return;
 	}
 
-	n = read(log->pfd, &log->buf[log->buf_used],
+	n = sys_read(log->pfd, &log->buf[log->buf_used],
 		 sizeof(log->buf) - log->buf_used);
 	if (n > 0) {
 		log->buf_used += n;
@@ -644,5 +644,44 @@ int ctdb_init_tevent_logging(struct ctdb_context *ctdb)
 	return ret;
 }
 
+void ctdb_collect_log(struct ctdb_context *ctdb, struct ctdb_get_log_addr *log_addr)
+{
+	TDB_DATA data;
 
-	
+	data = ctdb_log_ringbuffer_collect_log(ctdb, log_addr->level);
+
+	DEBUG(DEBUG_ERR,("Send log to %d:%d\n", (int)log_addr->pnn, (int)log_addr->srvid));
+	ctdb_client_send_message(ctdb, log_addr->pnn, log_addr->srvid, data);
+
+	if (data.dptr) {
+		talloc_free(data.dptr);
+	}
+}
+
+int32_t ctdb_control_get_log(struct ctdb_context *ctdb, TDB_DATA addr)
+{
+	struct ctdb_get_log_addr *log_addr = (struct ctdb_get_log_addr *)addr.dptr;
+	pid_t child;
+
+	/* spawn a child process to marshall the huge log blob and send it back
+	   to the ctdb tool using a MESSAGE
+	*/
+	child = ctdb_fork_no_free_ringbuffer(ctdb);
+	if (child == (pid_t)-1) {
+		DEBUG(DEBUG_ERR,("Failed to fork a log collector child\n"));
+		return -1;
+	}
+
+	if (child == 0) {
+		ctdb_set_process_name("ctdb_log_collector");
+		if (switch_from_server_to_client(ctdb, "log-collector") != 0) {
+			DEBUG(DEBUG_CRIT, (__location__ "ERROR: failed to switch log collector child into client mode.\n"));
+			_exit(1);
+		}
+		/* do logging here */
+		ctdb_collect_log(ctdb, log_addr);
+		_exit(0);
+	}
+
+	return 0;
+}
