@@ -534,7 +534,7 @@ static void set_recmode_handler(struct event_context *ev, struct fd_event *fde,
 	   the file   which at this time SHOULD be locked by the recovery
 	   daemon on the recmaster
 	*/		
-	ret = read(state->fd[0], &c, 1);
+	ret = sys_read(state->fd[0], &c, 1);
 	if (ret != 1 || c != 0) {
 		ctdb_request_control_reply(state->ctdb, state->c, NULL, -1, "managed to lock reclock file from inside daemon");
 		talloc_free(state);
@@ -623,7 +623,7 @@ int32_t ctdb_control_set_recmode(struct ctdb_context *ctdb,
 	/* force the databases to thaw */
 	for (i=1; i<=NUM_DB_PRIORITIES; i++) {
 		if (ctdb->freeze_handles[i] != NULL) {
-			ctdb_control_thaw(ctdb, i);
+			ctdb_control_thaw(ctdb, i, false);
 		}
 	}
 
@@ -679,11 +679,11 @@ int32_t ctdb_control_set_recmode(struct ctdb_context *ctdb,
 			cc = 1;
 		}
 
-		write(state->fd[1], &cc, 1);
+		sys_write(state->fd[1], &cc, 1);
 		/* make sure we die when our parent dies */
 		while (ctdb_kill(ctdb, parent, 0) == 0 || errno != ESRCH) {
 			sleep(5);
-			write(state->fd[1], &cc, 1);
+			sys_write(state->fd[1], &cc, 1);
 		}
 		_exit(0);
 	}
@@ -1092,8 +1092,7 @@ int32_t ctdb_control_try_delete_records(struct ctdb_context *ctdb, TDB_DATA inda
 	}	    
 
 
-	outdata->dptr = (uint8_t *)records;
-	outdata->dsize = talloc_get_size(records);
+	*outdata = ctdb_marshall_finish(records);
 
 	return 0;
 }
@@ -1290,9 +1289,7 @@ int32_t ctdb_control_receive_records(struct ctdb_context *ctdb,
 		rec = (struct ctdb_rec_data *)(rec->length + (uint8_t *)rec);
 	}
 
-
-	outdata->dptr = (uint8_t *)records;
-	outdata->dsize = talloc_get_size(records);
+	*outdata = ctdb_marshall_finish(records);
 
 	return 0;
 }
