@@ -364,7 +364,7 @@ static void ctdb_event_script_handler(struct event_context *ev, struct fd_event
 		return;
 	}
 
-	r = read(state->fd[0], &current->status, sizeof(current->status));
+	r = sys_read(state->fd[0], &current->status, sizeof(current->status));
 	if (r < 0) {
 		current->status = -errno;
 	} else if (r != sizeof(current->status)) {
@@ -754,8 +754,9 @@ static int ctdb_event_script_callback_v(struct ctdb_context *ctdb,
 		return -1;
 	}
 	if (!check_options(state->call, state->options)) {
-		DEBUG(DEBUG_ERR, ("Bad eventscript options '%s' for %s\n",
-				  ctdb_eventscript_call_names[state->call], state->options));
+		DEBUG(DEBUG_ERR, ("Bad eventscript options '%s' for '%s'\n",
+				  state->options,
+				  ctdb_eventscript_call_names[state->call]));
 		talloc_free(state);
 		return -1;
 	}
